module Css exposing
  ( Node
  , stylesheet, assign, declare, declares
  , style
  , (:-), (=>), (==>)
  )

{-| A library to create and use css stylesheets in Elm.

# Type Definitions
@docs Node

# Constructors
@docs stylesheet, assign, declare, declares

# Html Elements
@docs style

# Operators
@docs (:-), (=>), (==>)
-}

import Html exposing (Html)
import String


{-------------------------------------------------------------------------------
                               Type Definitions
-------------------------------------------------------------------------------}


{-| A Css stylesheet node. It can be a Rule, an @Rule, a Declaration, a
Selector...
-}
type alias Node =
  String


{-------------------------------------------------------------------------------
                                  Constructors
-------------------------------------------------------------------------------}


{-| Constructs an Html Style Element from a List of Node.
-}
stylesheet : List Node -> Html msg
stylesheet nodes =
  style [] (List.map (Html.text) nodes)


{-| Constructs a Declaration from a Property and a Value.
-}
assign : Node -> Node -> Node
assign property value =
    property ++ " : " ++ value ++ "; "


{-| Constructs a Block from a Head (Node) and a Body (List of Node).
-}
declare : Node -> List Node -> Node
declare head body =
  head ++ " { " ++ String.concat body ++ "} "


{-| Constructs a Block from a Head (List of Node) and a Body (List of Node).
-}
declares : List Node -> List Node -> Node
declares head body =
  String.join ", " head ++ " { " ++ String.concat body ++ "} "


{-------------------------------------------------------------------------------
                                 HTML Elements
-------------------------------------------------------------------------------}


{-| Html style tag.
-}
style : List (Html.Attribute msg) -> List (Html msg) -> Html msg
style =
  Html.node "style"


{-------------------------------------------------------------------------------
                                  Operators
-------------------------------------------------------------------------------}


{-| For Property :- Value. Alias of assign.
-}
(:-) : Node -> Node -> Node
(:-) = assign


{-| For Node => [Node]. Alias of declare.
-}
(=>) : Node -> List Node -> Node
(=>) = declare


{-| For [Selector] ==> [Declaration]. Alias of declares.
-}
(==>) : List Node -> List Node -> Node
(==>) = declares